/*****************************************************************************
*                                                                            *
*      COMMON OCEANOGRAPHIC DATA ACCESS SYSTEM (CODAS)                       *
*                                                                            *
*      WRITTEN BY:  RAMON CABRERA, ERIC FIRING, and JULIE RANADA             *
*                   JOINT INSTITUTE FOR MARINE AND ATMOSPHERIC RESEARCH      *
*                   1000 POPE ROAD  MSB 404                                  *
*                   HONOLULU, HI 96822                                       *
*                                                                            *
*      VERSION:     3.00                                                     *
*                                                                            *
*      DATE:        APRIL 1989                                               *
*                                                                            *
*****************************************************************************/
/*

    FILE:  io_.h

           General-purpose input-output routines.

*/
#ifndef io__included
#define io__included
#ifndef dbhost_included
#include "dbhost.h" /* PROTOTYPE_ALLOWED */
#endif
#if PROTOTYPE_ALLOWED
FILE *open_binary_file(char *filename_ptr, int accmode);
int EXISTS(char *filename_ptr);
int getline(FILE *fp, char *line_buffer);
int read_data(FILE *fp, char *data_add, long data_offset, unsigned int nb);
int write_data(FILE *fp, char *data_add, long data_offset, unsigned int nb);
int copy_data(FILE *fp1, FILE *fp2, long ofs1, long ofs2, unsigned int nb);
#else
FILE *open_binary_file();
int EXISTS();
int getline();
int read_data();
int write_data();
int copy_data();
#endif
#endif /* ifndef io__included */

